/* $Id: music.c,v 1.28 2010/09/10 22:55:50 lamonte Exp $
 *
 * music.c
 *
 * Additional acknowledgements, news, and updates to the Qodem Project
 * are available at http://qodem.sourceforge.net .
 *
 * This module is licensed under the GNU General Public License Version 2.
 * Please see the file "COPYING" in this directory for more information
 * about the GNU General Public License Version 2.
 *
 *     Copyright (C) 2003  Kevin Lamonte
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

#include "common.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/poll.h>
#include <ctype.h>
#include <unistd.h>
#include <assert.h>
#include "input.h"
#include "status.h"
#include "options.h"
#include "music.h"

#ifdef QODEM_USE_SDL
#include <SDL/SDL.h>
#include <SDL/SDL_audio.h>
#include <math.h>
#endif /* QODEM_USE_SDL */

/* #define DEBUG_MUSIC 1 */
#undef DEBUG_MUSIC
#ifdef DEBUG_MUSIC
#define MUSIC_DEBUG_FILE	"debug_music.txt"
static FILE * MUSIC_DEBUG_FILE_HANDLE = NULL;
#endif /* DEBUG_MUSIC */

/*
 * Each note is one semitone times the note prior.  The semitone value
 * is the 12th root of 2, i.e. 2^(1/12) .
 */
#define SEMITONE	1.05946309436

static float frequency_table[7][12];

#ifdef QODEM_USE_SDL

static int output_frequency = 11025;

/* Begin sine wave at x = 0 */
static int sdl_sine_wave_x = 0;

/* Frequency of current note */
static double sdl_Hz = 0;

static void sdl_callback(void * userdata, Uint8 * output, int output_max) {
	double pi = 3.1415926535;
	double A = 20;			/* Amplitude */
	double SR = output_frequency;
	double F = 2 * pi * sdl_Hz / SR;
	int i;

	for (i = 0; i < output_max; i++) {
		output[i] = (Uint8)(A * sin(F * sdl_sine_wave_x) + 128) & 0xFF;
		sdl_sine_wave_x++;
	}
}

#endif /* QODEM_USE_SDL */

/*
 * Initialize the frequency table
 */
void music_init() {
	int i, j;
	float current_tone;

#ifdef DEBUG_MUSIC
	if (MUSIC_DEBUG_FILE_HANDLE == NULL) {
		MUSIC_DEBUG_FILE_HANDLE = fopen(MUSIC_DEBUG_FILE, "w");
	}
	fprintf(MUSIC_DEBUG_FILE_HANDLE, "music_init()\n");
#endif /* DEBUG_MUSIC */

	/*
	 * Middle C is at the beginning of the third octave,
	 * i.e. frequency_table[2][0].
	 *
	 * Let's count down from two octaves below middle A:
	 *     A G# G F# F E D# D C# C  ==> 9 semitones down
	 *
	 */
	current_tone = 110.0f;
	for (i = 0; i < 9; i++) {
		current_tone /= SEMITONE;
	}

#ifdef DEBUG_MUSIC
	fprintf(MUSIC_DEBUG_FILE_HANDLE, "music_init(): first tone = %f hz\n", current_tone);
#endif /* DEBUG_MUSIC */

	/* Now count up */
	for (i=0; i<7; i++) {
		for (j=0; j<12; j++) {
			frequency_table[i][j] = current_tone;
			current_tone *= SEMITONE;
		}
	}

#ifdef DEBUG_MUSIC
	for (i=0; i<7; i++) {
		for (j=0; j<12; j++) {
			fprintf(MUSIC_DEBUG_FILE_HANDLE, "music_init(): [%d][%d] = %f hz\n", i, j, frequency_table[i][j]);
		}
	}
#endif /* DEBUG_MUSIC */

#ifdef QODEM_USE_SDL
	/* Cease sound */
	SDL_PauseAudio(1);

	/* TODO: handle the error here. */
	SDL_Init(SDL_INIT_AUDIO);

	SDL_AudioSpec spec;
	spec.freq = output_frequency;
	spec.format = AUDIO_U8;
	spec.channels = 1;
	spec.silence = 0;
	spec.samples = 8192;
	spec.padding = 0;
	spec.size = 0;
	spec.userdata = 0;

	spec.callback = sdl_callback;

	/* Cease sound */
	SDL_PauseAudio(1);

	/* TODO: handle the error here. */
	SDL_OpenAudio(&spec, NULL);

#endif /* QODEM_USE_SDL */

} /* ---------------------------------------------------------------------- */

/*
 * Shutdown SDL
 */
void music_teardown() {

#ifdef DEBUG_MUSIC
	fprintf(MUSIC_DEBUG_FILE_HANDLE, "music_teardown()\n");
#endif /* DEBUG_MUSIC */

#ifdef QODEM_USE_SDL
	/* Cease sound */
	SDL_PauseAudio(1);

	/* Shutdown audio */
	SDL_CloseAudio();

	/* Shutdown SDL */
	SDL_Quit();
#endif /* QODEM_USE_SDL */

} /* ---------------------------------------------------------------------- */

/*
 * Play a music sequence
 */
void play_music(const struct q_music_struct * music) {
	struct pollfd pfd;

#ifndef __linux
	/* Sound is not supported except on Linux console */
	q_status.sound = Q_FALSE;
#endif

	if (q_status.sound == Q_FALSE) {
		return;
	}

	while (music != NULL) {

#ifdef DEBUG_MUSIC
		fprintf(MUSIC_DEBUG_FILE_HANDLE, "play_music(): hertz = %d hz duration = %d millis\n", music->hertz, music->duration);
#endif /* DEBUG_MUSIC */

#ifdef QODEM_USE_SDL

		sdl_Hz = music->hertz;
		sdl_sine_wave_x = 0;

		/* Begin sound */
		SDL_PauseAudio(0);

#else


#ifdef __linux

		/*
		 * Linux can set the console beep with a weird CSI string
		 */
		if (music->hertz > 0) {
			/* A "note" */
			fprintf(stdout, "\033[10;%d]\033[11;%d]\007", music->hertz, music->duration);
			fflush(stdout);
		}

#endif /* __linux */

#endif /* QODEM_USE_SDL */

		pfd.fd = STDIN_FILENO;
		pfd.events = POLLIN;

		/* If the user presses a key, stop the sound */
		int rc = poll(&pfd, 1, music->duration);
		if (rc < 0) {
			/* Error */
		} else if (rc > 0) {

			/* Flush stdin */
			while ((rc = poll(&pfd, 1, 0)) > 0) {
				discarding_getch();
			}

			/* Break the outer while */
			break;
		} else {
			/* Timeout, we waited the whole duration  */
		}

		/* Next note */
		music = music->next;

	} /* while (music != NULL) */

#ifdef QODEM_USE_SDL

	/* Cease sound */
	SDL_PauseAudio(1);

#else

#ifdef __linux

	/* Restore the console beep.
	 *
	 * The linux defaults are in drivers/char/console.c, as of 2.4.22 it's
	 * 750 Hz 250 milliseconds.
	 */
	fprintf(stdout, "\033[10;750]\033[11;250]");
	fflush(stdout);

#endif /* __linux */

#endif /* QODEM_USE_SDL */

} /* ---------------------------------------------------------------------- */

/*
 * PLAY Statement
 *
 * Purpose:
 *
 * To play music by embedding a music macro language into the string data type.
 *
 * Syntax:
 *
 * PLAY string expression
 *
 * Comments:
 *
 * The single-character commands in PLAY are as follows:
 *
 *  A-G [#,+,-]   A-G are notes. # or + following a note produces a sharp;
 *                - produces a flat.  Any note followed by #,+,or - must refer
 *                to a black key on a piano.
 *
 *  L(n)          Sets the length of each note. L4 is a quarter note, L1 is a
 *                whole note, and so on. n may be from 1 to 64.
 *
 *                Length may also follow the note to change the length for that
 *                note only. A16 is equivalent to L16A.
 *
 *  MF            Music foreground. PLAY and SOUND statements are to run in
 *                foreground. That is, each subsequent note or sound is not
 *                started until the previous note or sound is finished. This
 *                is the initial default.
 *
 *  MB            Music background. PLAY and SOUND statements are to run in
 *                background. That is, each note or sound is placed in a buffer
 *                allowing the BASIC program to continue execution while music
 *                plays in the background. As many as 32 notes (or rests) can
 *                be played in background at one time.
 *
 *  MN            Music normal. Each note plays seven-eighths of the time
 *                determined by L (length).
 *
 *  ML            Music legato. Each note plays the full period set by L.
 *
 *  MS            Music staccato. Each note plays three-quarters of the time
 *                determined by L.
 *
 *  N(n)          Play note n. n may range from 0 to 84. In the 7 possible
 *                octaves, there are 84 notes. n set to 0 indicates a rest.
 *
 *  O(n)          Octave 0 sets the current octave. There are 7 octaves
 *                (0 through 6). Default is 4. Middle C is at the beginning
 *                of octave 3.
 *
 *  P(n)          Pause. P may range from 1-64.
 *
 *  T(n)          Tempo. T sets the number of L4s in a minute. n may range
 *                from 32-255. Default is 120.
 *
 *  . (period)    A period after a note increases the playing time of the note
 *                by 3/2 times the period determined by L (length of note)
 *                times T (tempo). Multiple periods can appear after  a note,
 *                and the playing time is scaled accordingly. For example, A.
 *                will cause the note A to play one and half times the playing
 *                time determined by L (length of the note) times T (the tempo);
 *                two periods placed after A (A..) will cause the note to be
 *                played at 9/4 times its ascribed value;  an A with three
 *                periods (A...) at 27/8, etc.
 *
 *                Periods may also appear after a P (pause), and increase the
 *                pause length as described above.
 *
 *  Xstring;      Executes a substring, where string is a variable assigned to a
 *                string of PLAY commands.
 *
 * Because of the slow clock interrupt rate, some notes do not play at higher
 * tempos;  for example, 1.64 at T255. These note/tempo combinations must be
 * determined through experimentation.
 *
 * >n             A greater-than symbol preceding the note n plays the note in
 *                the next higher octave.
 *
 * <n             A less-than symbol preceding the note n plays the note in the
 *                next lower octave.
 *
 * Note
 *
 * Numeric arguments follow the same syntax described under the DRAW  statement.
 *
 */

typedef enum {
	MUSIC_NONE,

	MUSIC_SOUND,

	MUSIC_L,
	MUSIC_M,
	MUSIC_N,
	MUSIC_O,
	MUSIC_P,
	MUSIC_T,

} MUSIC_STATE;

/* Default music options */
static int tempo = 120;
static int octave = 4;
static int length = 4;
/* normal, legato, stacatto */
static float style = 0.875;

/*
 * Play an ANSI music sequence
 */
void play_ansi_music(const unsigned char * buffer, const int buffer_n) {
	char * digits_end;

	int i;
	MUSIC_STATE state = MUSIC_NONE;

	struct q_music_struct music;
	struct q_music_struct * p;
	struct q_music_struct * q;

	/* For the '>' and '<' options */
	int note_octave = -1;

	/* For digits past the note */
	int note_length = 4;

	float note_length_multiplier = 1.0;

	/* Index into the second column of frequency_table (1..12) */
	int current_note = 0;

	if (q_status.sound == Q_FALSE) {
		return;
	}

	memset(&music, 0, sizeof(music));
	p = &music;

	/* Parse the GWBASIC/BASICA PLAY command string using a FSM */
	for (i = 0; i < buffer_n; i++) {

#ifdef DEBUG_MUSIC
		fprintf(MUSIC_DEBUG_FILE_HANDLE, "play_ansi_music(): state = %d note_octave = %d buffer[i] = %c\n", state, note_octave, buffer[i]);
#endif /* DEBUG_MUSIC */

music_top:

		switch (state) {

		case MUSIC_NONE:
			note_length = length;
			if (note_octave < 0) {
				note_octave = octave;
			}
			note_length_multiplier = 1.0;

			/* Looking for L, M, N, O, P, T, >, < */

			if (buffer[i] == 'L') {
				state = MUSIC_L;
				continue;
			} else if (buffer[i] == 'M') {
				state = MUSIC_M;
				continue;
			} else if (buffer[i] == 'N') {
				state = MUSIC_N;
				continue;
			} else if (buffer[i] == 'O') {
				state = MUSIC_O;
				continue;
			} else if (buffer[i] == 'P') {
				state = MUSIC_P;
				continue;
			} else if (buffer[i] == 'T') {
				state = MUSIC_T;
				continue;
			} else if (buffer[i] == '<') {
				note_octave = octave - 1;
				if (note_octave < 0) {
					note_octave = 0;
				}
				continue;
			} else if (buffer[i] == '>') {
				note_octave = octave + 1;
				if (note_octave > 6) {
					note_octave = 6;
				}
				continue;
			}

			if ((tolower(buffer[i]) >= 'a') && (tolower(buffer[i]) <= 'g')) {

				/* Set current note */
				switch (tolower(buffer[i])) {

				case 'c':
					current_note = 0;
					break;

				case 'd':
					current_note = 2;
					break;

				case 'e':
					current_note = 4;
					break;

				case 'f':
					current_note = 5;
					break;

				case 'g':
					current_note = 7;
					break;

				case 'a':
					current_note = 9;
					break;

				case 'b':
					current_note = 11;
					break;

				}

				state = MUSIC_SOUND;
			}

			break;

		case MUSIC_SOUND:

			if (((tolower(buffer[i]) >= 'a') && (tolower(buffer[i]) <= 'g')) ||
				(buffer[i] == '>') || (buffer[i] == '<')) {

				/* Play the old note */
				/* Play this note */
				p->hertz = frequency_table[note_octave][current_note];

				/*
				 * Duration (millis) = 1 /
				 *
				 *  (tempo/60) beat | (note_length/4) note | second
				 *  ------------------------------------------------------
				 *        second    |        beat          | 1000 millis
				 *
				 */
				p->duration = (int)((float)1000.0f / (((float)tempo / 60.0f) * ((float)note_length / 4.0f)));
				p->duration *= note_length_multiplier;

				q = (struct q_music_struct *)Xmalloc(sizeof(struct q_music_struct), __FILE__, __LINE__);
				memset(q, 0, sizeof(struct q_music_struct));
				p->next = q;
				q->hertz = 0;
				q->duration = p->duration * (1 - style);
				p->duration *= style;
				p = q;
				q = (struct q_music_struct *)Xmalloc(sizeof(struct q_music_struct), __FILE__, __LINE__);
				memset(q, 0, sizeof(struct q_music_struct));
				p->next = q;
				p = q;

				/* Reset the octave */
				note_octave = octave;
				note_length = length;
				note_length_multiplier = 1.0;

				if ((buffer[i] == '<') && (i + 1 == buffer_n)) {
					/* Error: string terminated on '<' */
					state = MUSIC_NONE;
					continue;
				} else if ((buffer[i] == '>') && (i + 1 == buffer_n)) {
					/* Error: string terminated on '>' */
					state = MUSIC_NONE;
					continue;
				} else if (buffer[i] == '<') {
					note_octave = octave - 1;
					if (note_octave < 0) {
						note_octave = 0;
					}
					/* Seek the next byte */
					i++;
				} else if (buffer[i] == '>') {
					note_octave = octave + 1;
					if (note_octave > 6) {
						note_octave = 6;
					}
					/* Seek the next byte */
					i++;
				}

				/* Set current_note to the new note */
				switch (tolower(buffer[i])) {

				case 'c':
					current_note = 0;
					break;

				case 'd':
					current_note = 2;
					break;

				case 'e':
					current_note = 4;
					break;

				case 'f':
					current_note = 5;
					break;

				case 'g':
					current_note = 7;
					break;

				case 'a':
					current_note = 9;
					break;

				case 'b':
					current_note = 11;
					break;

				}

				continue;
			}

			/* Looking for #, +, - */

			if (buffer[i] == '#') {
#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "SHARP\n");
#endif /* DEBUG_MUSIC */
				current_note++;
				continue;
			} else if (buffer[i] == '+') {
#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "SHARP\n");
#endif /* DEBUG_MUSIC */
				current_note++;
				continue;
			} else if (buffer[i] == '-') {
#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "FLAT\n");
#endif /* DEBUG_MUSIC */
				current_note--;
				continue;
			} else if (buffer[i] == '.') {
#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "DOT\n");
#endif /* DEBUG_MUSIC */
				/* Increase length by 50% */
				note_length_multiplier *= 1.5;
				continue;
			} else if ((buffer[i] >= '0') && (buffer[i] <= '9')) {
				/* Change duration of the note */
				note_length = strtol((char *)buffer + i, &digits_end, 10);

#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "new note length: %d\n", note_length);
#endif /* DEBUG_MUSIC */

				i = ((unsigned char *)digits_end - buffer) - 1;

				continue;
			}

			/* This must be some other option, so re-parse it.  First add the original note. */
			p->hertz = frequency_table[note_octave][current_note];

			/*
			 * Duration (millis) = 1 /
			 *
			 *  (tempo/60) beat | (note_length/4) note | second
			 *  ------------------------------------------------------
			 *        second    |        beat          | 1000 millis
			 *
			 */
			p->duration = (int)((float)1000.0f / (((float)tempo / 60.0f) * ((float)note_length / 4.0f)));
			p->duration *= note_length_multiplier;

			q = (struct q_music_struct *)Xmalloc(sizeof(struct q_music_struct), __FILE__, __LINE__);
			memset(q, 0, sizeof(struct q_music_struct));
			p->next = q;
			q->hertz = 0;
			q->duration = p->duration * (1 - style);
			p->duration *= style;
			p = q;
			q = (struct q_music_struct *)Xmalloc(sizeof(struct q_music_struct), __FILE__, __LINE__);
			memset(q, 0, sizeof(struct q_music_struct));
			p->next = q;
			p = q;

			state = MUSIC_NONE;
			goto music_top;

		case MUSIC_M:

			/* Looking for F, B, L, N, S */
			if ((tolower(buffer[i]) == 'f') || (tolower(buffer[i]) == 'b')) {
#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "MUSIC FOREGROUND/BACKGROUND\n");
#endif /* DEBUG_MUSIC */
				state = MUSIC_NONE;
				break;
			}

			/* Normal */
			if (tolower(buffer[i]) == 'n') {
#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "MUSIC NORMAL\n");
#endif /* DEBUG_MUSIC */
				style = 0.875;
				state = MUSIC_NONE;
				break;
			}

			/* Legato */
			if (tolower(buffer[i]) == 'l') {
#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "MUSIC LEGATO\n");
#endif /* DEBUG_MUSIC */
				style = 1.000;
				state = MUSIC_NONE;
				break;
			}

			/* Staccato */
			if (tolower(buffer[i]) == 's') {
#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "MUSIC STACATTO\n");
#endif /* DEBUG_MUSIC */
				style = 0.750;
				state = MUSIC_NONE;
				break;
			}

			/* This must be some other option, so re-parse it */
			state = MUSIC_NONE;
			goto music_top;

		case MUSIC_L:

			/* Looking for digits */
			if ((buffer[i] >= '0') && (buffer[i] <= '9')) {
				length = strtol((char *)buffer + i, &digits_end, 10);

#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "new length: %d\n", length);
#endif /* DEBUG_MUSIC */

				/* i += (int)digits_end - (int)(buffer) - i - 1; */
				i = ((unsigned char *)digits_end - buffer) - 1;

			} else {
				/* Syntax error, bail out */
				goto music_done;
			}

			state = MUSIC_NONE;
			break;

		case MUSIC_N:

			/* Looking for digits */
			if ((buffer[i] >= '0') && (buffer[i] <= '9')) {
				current_note = strtol((char *)buffer + i, &digits_end, 10);

#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "RAW current_note: %d\n", current_note);
#endif /* DEBUG_MUSIC */

				/* i += (int)digits_end - (int)(buffer) - i - 1; */
				i = ((unsigned char *)digits_end - buffer) - 1;

				/* Play this note */
				p->hertz = frequency_table[current_note / 12][current_note % 12];

				/*
				 * Duration (millis) = 1 /
				 *
				 *  (tempo/60) beat | (note_length/4) note | second
				 *  ------------------------------------------------------
				 *        second    |        beat          | 1000 millis
				 *
				 */
				p->duration = (int)((float)1000.0f / (((float)tempo / 60.0f) * ((float)note_length / 4.0f)));
				p->duration *= note_length_multiplier;

				q = (struct q_music_struct *)Xmalloc(sizeof(struct q_music_struct), __FILE__, __LINE__);
				memset(q, 0, sizeof(struct q_music_struct));
				p->next = q;
				q->hertz = 0;
				q->duration = p->duration * (1 - style);
				p->duration *= style;
				p = q;
				q = (struct q_music_struct *)Xmalloc(sizeof(struct q_music_struct), __FILE__, __LINE__);
				memset(q, 0, sizeof(struct q_music_struct));
				p->next = q;
				p = q;

			} else {
				/* Syntax error, bail out */
				goto music_done;
			}

			state = MUSIC_NONE;
			break;

		case MUSIC_O:

			/* Looking for digits */
			if ((buffer[i] >= '0') && (buffer[i] <= '9')) {
				octave = strtol((char *)buffer + i, &digits_end, 10);

#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "new octave: %d\n", octave);
#endif /* DEBUG_MUSIC */
				/* i += (int)digits_end - (int)(buffer) - i - 1; */
				i = ((unsigned char *)digits_end - buffer) - 1;

				if ((octave < 0) || (octave > 6)) {
					/* Invalid octave, reset to default */
					octave = 4;
				}
				note_octave = octave;

			} else {
				/* Syntax error, bail out */
				goto music_done;
			}

			state = MUSIC_NONE;
			break;

		case MUSIC_P:


			/* Looking for . or digits */
			if (buffer[i] == '.') {
#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "DOT\n");
#endif /* DEBUG_MUSIC */
				/* Increase length by 50% */
				note_length_multiplier *= 1.5;
				continue;

			} else if ((buffer[i] >= '0') && (buffer[i] <= '9')) {
				/* Change duration of the note */
				note_length = strtol((char *)buffer + i, &digits_end, 10);

#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "new note length: %d\n", note_length);
#endif /* DEBUG_MUSIC */

				/* i += (int)digits_end - (int)(buffer) - i - 1; */
				i = ((unsigned char *)digits_end - buffer) - 1;

				continue;
			}

			/* Play the pause */
			p->hertz = 0;
			/*
			 * Duration (millis) = 1 /
			 *
			 *  (tempo/60) beat | (note_length/4) note | second
			 *  ------------------------------------------------------
			 *        second    |        beat          | 1000 millis
			 *
			 */
			p->duration = (int)((float)1000.0f / (((float)tempo / 60.0f) * ((float)note_length / 4.0f)));
			p->duration *= note_length_multiplier;
			q = (struct q_music_struct *)Xmalloc(sizeof(struct q_music_struct), __FILE__, __LINE__);
			memset(q, 0, sizeof(struct q_music_struct));
			p->next = q;
			p = q;

			/* This must be some other option, so re-parse it */
			state = MUSIC_NONE;
			goto music_top;

		case MUSIC_T:

			/* Looking for digits */
			if ((buffer[i] >= '0') && (buffer[i] <= '9')) {
				tempo = strtol((char *)buffer + i, &digits_end, 10);

#ifdef DEBUG_MUSIC
				fprintf(MUSIC_DEBUG_FILE_HANDLE, "new tempo: %d\n", tempo);
#endif /* DEBUG_MUSIC */

				/* i += (int)digits_end - (int)(buffer) - i - 1; */
				i = ((unsigned char *)digits_end - buffer) - 1;

			} else {
				/* Syntax error, bail out */
				goto music_done;
			}

			state = MUSIC_NONE;
			break;


		} /* switch (state) */
	}

 music_done:

	/* See if we've got one more to go */
	if (state == MUSIC_SOUND) {

		/* Play the old note */
		/* Play this note */
		p->hertz = frequency_table[note_octave][current_note];

		/*
		 * Duration (millis) = 1 /
		 *
		 *  (tempo/60) beat | (note_length/4) note | second
		 *  ------------------------------------------------------
		 *        second    |        beat          | 1000 millis
		 *
		 */
		p->duration = (int)((float)1000.0f / (((float)tempo / 60.0f) * ((float)note_length / 4.0f)));
		p->duration *= note_length_multiplier;

		q = (struct q_music_struct *)Xmalloc(sizeof(struct q_music_struct), __FILE__, __LINE__);
		memset(q, 0, sizeof(struct q_music_struct));
		p->next = q;
		q->hertz = 0;
		q->duration = p->duration * (1 - style);
		p->duration *= style;
		p = q;
		q = (struct q_music_struct *)Xmalloc(sizeof(struct q_music_struct), __FILE__, __LINE__);
		memset(q, 0, sizeof(struct q_music_struct));
		p->next = q;
		p = q;
	}

	/* Play the sequence */
	play_music(&music);

	/* Cleanup memory */
	p = music.next;
	while (p != NULL) {
		q = p->next;
		Xfree(p, __FILE__, __LINE__);
		p = q;
	}

} /* ---------------------------------------------------------------------- */

/*
 * Play one of the defined music sequences for a normal event
 */
void play_sequence(const Q_MUSIC_SEQUENCE sequence) {
	unsigned char music_sequence[OPTIONS_LINE_SIZE];

	if (q_status.sound == Q_FALSE) {
		return;
	}

	switch (sequence) {

	case Q_MUSIC_CONNECT:
		strncpy((char *)music_sequence, get_option(Q_OPTION_MUSIC_CONNECT), sizeof(music_sequence) - 1);
		music_sequence[sizeof(music_sequence) - 1] = 0;
		if (strcmp((char *)music_sequence, "none") != 0) {
			play_ansi_music(music_sequence, strlen((char *)music_sequence));
		}
		break;

	case Q_MUSIC_CONNECT_MODEM:
		strncpy((char *)music_sequence, get_option(Q_OPTION_MUSIC_CONNECT_MODEM), sizeof(music_sequence) - 1);
		music_sequence[sizeof(music_sequence) - 1] = 0;
		if (strcmp((char *)music_sequence, "none") != 0) {
			play_ansi_music(music_sequence, strlen((char *)music_sequence));
		}
		break;

	case Q_MUSIC_UPLOAD:
		strncpy((char *)music_sequence, get_option(Q_OPTION_MUSIC_UPLOAD), sizeof(music_sequence) - 1);
		music_sequence[sizeof(music_sequence) - 1] = 0;
		play_ansi_music(music_sequence, strlen((char *)music_sequence));
		break;

	case Q_MUSIC_DOWNLOAD:
		strncpy((char *)music_sequence, get_option(Q_OPTION_MUSIC_DOWNLOAD), sizeof(music_sequence) - 1);
		music_sequence[sizeof(music_sequence) - 1] = 0;
		play_ansi_music(music_sequence, strlen((char *)music_sequence));
		break;

	default:
		/* Should never get here */
		assert(1 == 0);

	}

} /* ---------------------------------------------------------------------- */
